/**
 * https://github.com/image-size/image-size/blob/main/LICENSE
 * @license MIT
 *
 * Copyright (c) 2016-23 [these people](https://github.com/image-size/image-size/graphs/contributors)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
const parseWebp = (data) => {
  const readUInt16LE = (input, offset2) => input[offset2] + (input[offset2 + 1] << 8);
  const readUInt24LE = (input, offset2) => input[offset2] + (input[offset2 + 1] << 8) + (input[offset2 + 2] << 16);
  const toUTF8String = (input, offset2, length) => new TextDecoder().decode(input.subarray(offset2, offset2 + length));
  const calculateExtended = (input) => ({
    width: 1 + readUInt24LE(input, 4),
    height: 1 + readUInt24LE(input, 7)
  });
  const calculateLossless = (input) => ({
    width: 1 + (input[1] | (input[2] & 63) << 8),
    height: 1 + ((input[2] & 240) >> 4 | input[3] << 4 | (input[4] & 3) << 12)
  });
  const calculateLossy = (input) => ({
    width: readUInt16LE(input, 6) & 16383,
    height: readUInt16LE(input, 8) & 16383
  });
  if (!("RIFF" === toUTF8String(data, 0, 4) && "WEBP" === toUTF8String(data, 8, 4))) {
    return false;
  }
  let offset = 12;
  while (offset < data.length) {
    const chunkType = toUTF8String(data, offset, 4);
    const chunkSize = readUInt24LE(data, offset + 4);
    offset += 8;
    switch (chunkType) {
      case "VP8X":
        return {
          ...calculateExtended(data.subarray(offset, offset + chunkSize))
        };
      case "VP8 ":
        return {
          ...calculateLossy(data.subarray(offset, offset + chunkSize))
        };
      case "VP8L":
        return {
          ...calculateLossless(data.subarray(offset, offset + chunkSize))
        };
    }
    offset += chunkSize;
  }
  return false;
};
const parseHeif = (data) => {
  const toUTF8String = (input, start = 0, end = input.length) => new TextDecoder().decode(input.slice(start, end));
  const readUInt32BE = (input, offset = 0) => input[offset] * 16777216 + input[offset + 1] * 65536 + input[offset + 2] * 256 + input[offset + 3];
  function readBox(buffer, offset) {
    if (buffer.length - offset < 4)
      return;
    const boxSize = readUInt32BE(buffer, offset);
    if (buffer.length - offset < boxSize)
      return;
    return {
      name: toUTF8String(buffer, 4 + offset, 8 + offset),
      offset,
      size: boxSize
    };
  }
  function findBox(buffer, boxName, offset) {
    while (offset < buffer.length) {
      const box = readBox(buffer, offset);
      if (!box)
        break;
      if (box.name === boxName)
        return box;
      offset += box.size;
    }
  }
  const metaBox = findBox(data, "meta", 0);
  if (!metaBox)
    return false;
  const iprpBox = findBox(data, "iprp", metaBox.offset + 12);
  if (!iprpBox)
    return false;
  const ipcoBox = findBox(data, "ipco", iprpBox.offset + 8);
  if (!ipcoBox)
    return false;
  const ispeBox = findBox(data, "ispe", ipcoBox.offset + 8);
  if (!ispeBox)
    return false;
  const width = readUInt32BE(data, ispeBox.offset + 16);
  const height = readUInt32BE(data, ispeBox.offset + 12);
  return { width, height };
};
const parseTiff = (data) => {
  const readUInt16 = (buffer, offset, isBigEndian2) => isBigEndian2 ? buffer[offset] * 256 + buffer[offset + 1] : buffer[offset + 1] * 256 + buffer[offset];
  const readUInt32 = (buffer, offset, isBigEndian2) => {
    const value = isBigEndian2 ? buffer[offset] << 24 | buffer[offset + 1] << 16 | buffer[offset + 2] << 8 | buffer[offset + 3] : buffer[offset + 3] << 24 | buffer[offset + 2] << 16 | buffer[offset + 1] << 8 | buffer[offset];
    return value >>> 0;
  };
  const isBigEndian = data[0] === 77 && data[1] === 77;
  const ifdOffset = readUInt32(data, 4, isBigEndian);
  const entries = readUInt16(data, ifdOffset, isBigEndian);
  let width = 0, height = 0;
  for (let i = 0; i < entries; i++) {
    const entryOffset = ifdOffset + 2 + i * 12;
    const tag = readUInt16(data, entryOffset, isBigEndian);
    const type = readUInt16(data, entryOffset + 2, isBigEndian);
    if (tag === 256 && type === 3) {
      width = readUInt16(data, entryOffset + 8, isBigEndian);
    } else if (tag === 257 && type === 3) {
      height = readUInt16(data, entryOffset + 8, isBigEndian);
    }
  }
  if (!width || !height) {
    return false;
  }
  return { width, height };
};
const parseSvg = (data) => {
  const text = new TextDecoder("utf-8").decode(data);
  const widthRegex = /width="([^"]+)"/;
  const heightRegex = /height="([^"]+)"/;
  const viewBoxRegex = /viewBox="([^"]+)"/;
  const widthMatch = text.match(widthRegex);
  const heightMatch = text.match(heightRegex);
  const viewBoxMatch = text.match(viewBoxRegex);
  let width = null;
  let height = null;
  if (viewBoxMatch) {
    const values = viewBoxMatch[1].split(" ");
    width = parseFloat(values[2]);
    height = parseFloat(values[3]);
  } else {
    width = widthMatch ? parseFloat(widthMatch[1]) : null;
    height = heightMatch ? parseFloat(heightMatch[1]) : null;
  }
  if (width !== null && height !== null) {
    return { width, height };
  } else {
    return false;
  }
};
const parseGif = (data) => {
  try {
    const width = data[6] | data[7] << 8;
    const height = data[8] | data[9] << 8;
    return {
      width,
      height
    };
  } catch {
    return false;
  }
};
const parsePng = (data) => {
  try {
    let chunkNameOffset = 12;
    let widthHeightOffset = 16;
    const chunkName = new TextDecoder("utf-8").decode(
      data.slice(chunkNameOffset, chunkNameOffset + 4)
    );
    if (chunkName === "CgBI") {
      chunkNameOffset = 28;
      widthHeightOffset = 32;
    }
    const width = data[widthHeightOffset] << 24 | data[widthHeightOffset + 1] << 16 | data[widthHeightOffset + 2] << 8 | data[widthHeightOffset + 3];
    const height = data[widthHeightOffset + 4] << 24 | data[widthHeightOffset + 5] << 16 | data[widthHeightOffset + 6] << 8 | data[widthHeightOffset + 7];
    return {
      width,
      height
    };
  } catch {
    return false;
  }
};
const parseJpeg = (data) => {
  try {
    const SOF0 = 192;
    const SOF2 = 194;
    let position = 2;
    while (position < data.length) {
      const marker = data[position + 1];
      const blockLength = data[position + 2] << 8 | data[position + 3];
      if (marker === SOF0 || marker === SOF2) {
        const height = data[position + 5] << 8 | data[position + 6];
        const width = data[position + 7] << 8 | data[position + 8];
        return {
          width,
          height
        };
      }
      position += blockLength + 2;
    }
  } catch {
    return false;
  }
  return false;
};
const parseBmp = (data) => {
  try {
    const readUInt32LE = (input, offset) => input[offset] | input[offset + 1] << 8 | input[offset + 2] << 16 | input[offset + 3] << 24;
    const readInt32LE = (input, offset) => (input[offset] | input[offset + 1] << 8 | input[offset + 2] << 16 | input[offset + 3] << 24) << 0;
    const width = readUInt32LE(data, 18);
    const height = Math.abs(readInt32LE(data, 22));
    return { width, height };
  } catch (error) {
    return false;
  }
};
const parseIco = (data) => {
  const readUInt16LE = (input, offset) => input[offset] | input[offset + 1] << 8;
  const TYPE_ICON = 1;
  const SIZE_HEADER = 6;
  const SIZE_IMAGE_ENTRY = 16;
  function getSizeFromOffset(input, offset) {
    const value = input[offset];
    return value === 0 ? 256 : value;
  }
  function getImageSize(input, imageIndex) {
    const offset = SIZE_HEADER + imageIndex * SIZE_IMAGE_ENTRY;
    const width = getSizeFromOffset(input, offset);
    const height = getSizeFromOffset(input, offset + 1);
    return { width, height };
  }
  try {
    const reserved = readUInt16LE(data, 0);
    const type = readUInt16LE(data, 2);
    const imageCount = readUInt16LE(data, 4);
    if (reserved !== 0 || type !== TYPE_ICON || imageCount === 0) {
      return false;
    }
    return getImageSize(data, 0);
  } catch (error) {
    return false;
  }
};
const parseFunctionMap = {
  "image/webp": parseWebp,
  "image/heic": parseHeif,
  "image/heif": parseHeif,
  "image/avif": parseHeif,
  "image/tiff": parseTiff,
  "image/svg+xml": parseSvg,
  "image/gif": parseGif,
  "image/png": parsePng,
  "image/jpeg": parseJpeg,
  "image/bmp": parseBmp,
  "image/vnd.microsoft.icon": parseIco,
  "image/x-icon": parseIco
};
async function probe(url) {
  const response = await fetch(url);
  if (!response.body || !response.ok) {
    throw new Error("Failed to fetch image");
  }
  const reader = response.body.getReader();
  const contentType = response.headers.get("Content-Type") || "";
  const parseFunction = parseFunctionMap[contentType];
  if (!parseFunction) {
    await reader.cancel();
    throw new Error("Unsupported file type");
  }
  let done, value;
  let accumulatedChunks = new Uint8Array();
  while (!done) {
    const readResult = await reader.read();
    done = readResult.done;
    if (done)
      break;
    if (readResult.value) {
      value = readResult.value;
      let tmp = new Uint8Array(accumulatedChunks.length + value.length);
      tmp.set(accumulatedChunks, 0);
      tmp.set(value, accumulatedChunks.length);
      accumulatedChunks = tmp;
      const result = parseFunction(accumulatedChunks);
      if (result !== false) {
        await reader.cancel();
        return result;
      }
    }
  }
  throw new Error("Failed to parse the size");
}
export {
  probe
};
